#  Copyright (c) New Cloud Technologies, Ltd. 2013-2022.
#  Author: Vitaly Isaev <vitaly.isaev@myoffice.team>
#  License: https://github.com/newcloudtechnologies/rbfopt-go/blob/master/LICENSE

from typing import Final

Cost: Final = "cost"
InvalidParameterCombination: Final = "invalid_parameter_combination"
Iteration: Final = "iteration"
